#include "mymodel.h"

#include <QtCore/QDebug>
#include <QtCore/QTextCodec>

#include <QtGui/QApplication>
#include <QtGui/QComboBox>
#include <QtGui/QMessageBox>

#include <QtSql/QSqlDatabase>
#include <QtSql/QSqlError>
#include <QtSql/QSqlQuery>

#ifdef Q_OS_WIN

static void displayError(const QString &errorText)
{
    QMessageBox::critical(0, QApplication::applicationName(), errorText);
}

#else

static void displayError(const QString &errorText)
{
    qCritical("%s", qPrintable(errorText));
}

#endif

const char *const SQL_DRIVER = "QSQLITE";

static bool initDatabase()
{
    if (!QSqlDatabase::isDriverAvailable(QLatin1String(SQL_DRIVER))) {
        qDebug() << QObject::trUtf8("initDatabase(): %1 not found").arg(SQL_DRIVER);
        return false;
    }

    QSqlDatabase database = QSqlDatabase::addDatabase(QLatin1String(SQL_DRIVER));
    database.setDatabaseName(QLatin1String(":memory:"));

    if (!database.open()) {
        qDebug() << QObject::trUtf8("initDatabase(): database open error: %1").arg(database.lastError().text());
        return false;
    }

    return true;
}

static bool createTable()
{
    QSqlDatabase database = QSqlDatabase::database();

    if (!database.isOpen()) {
        if (!database.open()) {
            qDebug() << QObject::trUtf8("createTable(): database open error: %1").arg(database.lastError().text());
            return false;
        }
    }

    QSqlQuery query(database);

    const QString queryText = QLatin1String("CREATE TABLE colors (id INTEGER NOT NULL, color VARCHAR(30) NULL)");

    if (!query.prepare(queryText)) {
        qDebug() << QObject::trUtf8("createTable(): query prepare error: %1").arg(query.lastError().text());
        return false;
    }

    if (!query.exec()) {
        qDebug() << QObject::trUtf8("createTable(): query execution error: %1").arg(query.lastError().text());
        return false;
    }

    return true;
}

static bool generateData()
{
    QSqlDatabase database = QSqlDatabase::database();

    if (!database.isOpen()) {
        if (!database.open()) {
            qDebug() << QObject::trUtf8("generateData(): database open error: %1").arg(database.lastError().text());
            return false;
        }
    }

    QSqlQuery query(database);

    const QString queryText = QLatin1String("INSERT INTO colors (id, color)"
                                            "VALUES (?, ?)");

    if (!query.prepare(queryText)) {
        qDebug() << QObject::trUtf8("generateData(): query prepare error: %1").arg(query.lastError().text());
        return false;
    }

    QStringList colorsList;
    colorsList.append(QObject::trUtf8("Красный"));
    colorsList.append(QObject::trUtf8("Жёлтый"));
    colorsList.append(QObject::trUtf8("Зелёный"));

    for (int i = 0; i < colorsList.size(); ++i) {
        query.addBindValue(i);
        query.addBindValue(colorsList.at(i));

        if (!query.exec()) {
            qDebug() << QObject::trUtf8("generateData(): query execution error: %1").arg(query.lastError().text());
            return false;
        }
    }

    return true;
}

int main(int argc, char *argv[])
{
    QTextCodec::setCodecForTr(QTextCodec::codecForName("UTF-8"));
    QTextCodec::setCodecForCStrings(QTextCodec::codecForName("UTF-8"));

    QApplication a(argc, argv);

    if (!initDatabase()) {
        const QString &errorText = QObject::trUtf8("Не могу открыть базу данных");
        displayError(errorText);
        return 1;
    }

    if (!createTable()) {
        const QString &errorText = QObject::trUtf8("Не могу создать таблицу в базе данных");
        displayError(errorText);
        return 1;
    }

    if (!generateData()) {
        const QString &errorText = QObject::trUtf8("Не могу вставить данные в базу данных");
        displayError(errorText);
        return 1;
    }

    MyModel model;

    if (!model.select()) {
        const QString &errorText = QObject::trUtf8("Не могу получить данные из базы данных");
        displayError(errorText);
        return 1;
    }

    QComboBox comboBox;
    comboBox.setModel(&model);
    comboBox.setModelColumn(MY_MODEL_COLOR_COLUMN);
    comboBox.show();

    return a.exec();
}
